﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using JsonApiServer.Attributes;
using JsonApiServer.Models;
using Newtonsoft.Json;
using log4net;

namespace JsonApiServer.Controllers
{
   /// <summary>
   /// The end user (reader) controller
   /// </summary>
   public class ReaderController : ApiController
   {
      public enum SortingOrder
      {
         Ascending = 1,
         Descending = -1
      }

      private const int UserCount = 25;
      private static readonly List<Reader> Readers;

      private readonly ILog _log = LogManager.GetLogger(typeof(ReaderController));

      static ReaderController()
      {
         // remove this
         Readers = new List<Reader>();
         for (int i = 1; i <= UserCount; i++)
         {
            Readers.Add(new Reader
            {
               Id = string.Format("{0}", i),
               Username = string.Format("user{0}@domain.com", i),
               IsActive = true,
               AccessPolicy = new AccessPolicy()
            });
         }

         Readers.Add(new Reader()
         {
            Id = "andyw@vitrium.com",
            Username = "andyw@vitrium.com",
            IsActive = true,
            AccessPolicy = new AccessPolicy()
         });

         Readers.Add(new Reader()
         {
            Id = "stevenc@vitrium.com",
            Username = "stevenc@vitrium.com",
            IsActive = true,
            AccessPolicy = new AccessPolicy()
         });
      }

      /// <summary>
      /// Retrieve paged sets of readers
      /// </summary>
      /// <param name="page"></param>
      /// <param name="filter"></param>
      /// <param name="sort"></param>
      /// <returns></returns>
      [Route("api/2.0/readers")]
      [BindJson(typeof(Page), "page")]
      [BindJson(typeof(Filter), "filter")]
      [BindJson(typeof(Sort), "sort")]
      public PagedList<Reader> Get([FromUri] Page page, [FromUri] Filter filter, [FromUri] Sort sort)
      {
         _log.Debug("Get readers begins");

         // change this method to get readers
         IQueryable<Reader> results = GetAllReaders();

         if (filter != null)
         {
            _log.Debug("filter: " + JsonConvert.SerializeObject(filter));
            results = results.Where(r => r.Username.ToLower().Contains(filter.Contains.ToLower()));
         }

         int totalRecords = results.Count();

         if (page != null)
         {
            _log.Debug("page:" + JsonConvert.SerializeObject(page));
            results = results.Skip((page.Index - 1) * page.Size).Take(page.Size);
         }

         if (sort != null)
         {
            _log.Debug("sort:" + JsonConvert.SerializeObject(sort));
            if (sort.Username == (int)SortingOrder.Ascending)
            {
               results = results.OrderBy(r => r.Username);
            }
            else
            {
               results = results.OrderByDescending(r => r.Username);
            }
         }

         var result = new PagedList<Reader>
         {
            Results = results.ToList(),
            TotalRecords = totalRecords
         };

         _log.Debug("Results:" + JsonConvert.SerializeObject(result));
         _log.Debug("Get readers ends");

         return result;
      }

      /// <summary>
      /// retrieve a reader by id
      /// </summary>
      /// <param name="id"></param>
      /// <returns></returns>
      public Reader Get(string id)
      {
         _log.Debug("Get reader by id begins");
         _log.Debug(string.Format("userId: {0}", id));
         if (string.IsNullOrEmpty(id))
            throw new HttpResponseException(Request.CreateErrorResponse(HttpStatusCode.NotFound, "this item does not exist"));

         //change this to get reader by Id
         var reader = Readers.FirstOrDefault(r => r.Id == id.ToString(CultureInfo.InvariantCulture));
         _log.Debug("Results:" + JsonConvert.SerializeObject(reader));
         _log.Debug("Get reader by id ends");

         return reader;
      }

      /// <summary>
      /// retrieve a reader by username
      /// </summary>
      /// <param name="username"></param>
      /// <returns></returns>
      [Route("api/2.0/reader")]
      public Reader GetByUserName(string username)
      {
         _log.Debug("Get reader by user name begins");
         _log.Debug(string.Format("user name: {0}", username));

         if (string.IsNullOrEmpty(username))
            throw new HttpResponseException(Request.CreateErrorResponse(HttpStatusCode.NotFound, "this item does not exist"));

         //change this to get reader by username
         var reader = Readers.FirstOrDefault(r => string.Compare(r.Username, username, StringComparison.OrdinalIgnoreCase) == 0);

         _log.Debug("Results:" + JsonConvert.SerializeObject(reader));
         _log.Debug("Get reader by user name ends");

         return reader;
      }

      /// <summary>
      /// retrieve a reader by single sign-on token
      /// </summary>
      /// <param name="token"></param>
      /// <returns></returns>
      [Route("api/2.0/reader")]
      public Reader GetByToken(string token)
      {
         _log.Debug("Get reader by token begins");
         _log.Debug(string.Format("user name: {0}", token));

         if (string.IsNullOrEmpty(token))
            throw new HttpResponseException(Request.CreateErrorResponse(HttpStatusCode.NotFound, "this item does not exist"));

         // change this to get reader by token
         if (string.CompareOrdinal(token, "3BBD5CC2-954A-40A1-8653-C14D8B4B4824") == 0)
         {
            var reader = Readers.FirstOrDefault(x => x.Id == "8");
            _log.Debug("Results:" + JsonConvert.SerializeObject(reader));
            _log.Debug("Get reader by token ends");
            return reader;
         }

         var exception = new HttpResponseException(Request.CreateErrorResponse(HttpStatusCode.NotFound, "this item does not exist"));
         _log.Error("user not found", exception);

         throw exception;
      }

      private IQueryable<Reader> GetAllReaders()
      {
         return Readers.AsQueryable();
      }

      public class Filter
      {
         public string Contains { get; set; }
      }

      public class Page
      {
         public int Index { get; set; }
         public int Size { get; set; }
      }

      public class Sort
      {
         public int Username { get; set; }
      }
   }
}